# Matmul算子运行验证<a name="ZH-CN_TOPIC_0302083283"></a>

## 功能描述<a name="section1421916179418"></a>

该样例实现了对[自定义算子MatmulTik](../../1_custom_op/doc/Matmul_CN.md)的功能验证，通过将自定义算子转换为单算子离线模型文件，然后通过AscendCL加载单算子模型文件进行运行。

说明：单算子模型文件的生成只依赖算子代码实现文件、算子原型定义、算子信息库，不依赖算子适配插件。

## 目录结构<a name="section8733528154320"></a>

```
├── inc                           //头文件目录
│   ├── common.h                  // 声明公共方法类，用于读取二进制文件
│   ├── operator_desc.h          //算子描述声明文件，包含算子输入/输出，算子类型以及输入描述与输出描述
│   ├── op_runner.h              //算子运行相关信息声明文件，包含算子输入/输出个数，输入/输出大小等
├── run                           // 单算子执行需要的文件存放目录
│   ├── out    // 单算子执行需要的可执行文件存放目录
│        └── test_data         // 测试数据存放目录
│           ├── config
│               └── acl.json       //用于进行~acl初始化，请勿修改此文件
│               └── matmul_tik_op.json    // 算子描述文件，用于构造单算子模型文件
│           ├── data
│               └── check_out.py    // 生成期望结果并与实际结果进行比对的脚本
│               └── generate_datatik.py    // 生成测试数据的脚本
├── src
│   ├── CMakeLists.txt    // 编译规则文件
│   ├── common.cpp         // 公共函数，读取二进制文件函数的实现文件  
│   ├── main.cpp    // 将单算子编译为om文件并加载om文件执行，此文件中包含算子的相关配置，若验证其他单算子，基于此文件修改
│   ├── operator_desc.cpp     // 构造算子的输入与输出描述
│   ├── op_runner.cpp   // 单算子编译与运行函数实现文件
```

## 环境要求<a name="zh-cn_topic_0230709958_section1256019267915"></a>

-   操作系统及架构：CentOS x86\_64、CentOS aarch64、Ubuntu 18.04 x86\_64
-   芯片：Ascend 310、Ascend 710
-   python及依赖的库：python3.7.5
-   已完成昇腾AI软件栈的部署。
-   已参考[custom\_op](../../1_custom_op)完成自定义算子的编译部署。

## 配置环境变量<a name="section053142383519"></a>

-   Ascend 310 EP
    1.  开发环境上，设置生成单算子离线模型的环境变量。

        环境变量配置示例如下：

        ```
        export install_path=$HOME/Ascend/ascend-toolkit/latest
        export PATH=/usr/local/python3.7.5/bin:${install_path}/atc/ccec_compiler/bin:${install_path}/atc/bin:$PATH  # 如果用户环境存在多个python3版本，则指定使用的python3.7.5版本，python3.7.5安装路径请根据实际情况替换；同时设定ATC工具可执行文件所在路径
        export ASCEND_OPP_PATH=${install_path}/opp
        ```

        “$HOME/Ascend”请替换“Ascend-cann-toolkit”包的实际安装路径。

    2.  开发环境上，设置环境变量，配置AscendCL单算子验证程序编译依赖的头文件与库文件路径。

        编译脚本会按环境变量指向的路径查找编译依赖的头文件和库文件，“$HOME/Ascend”请替换“Ascend-cann-toolkit”包的实际安装路径。

        -   当运行环境操作系统架构是x86时，配置示例如下所示：

            ```
            export DDK_PATH=$HOME/Ascend/ascend-toolkit/latest/x86_64-linux
            export NPU_HOST_LIB=$HOME/Ascend/ascend-toolkit/latest/x86_64-linux/fwkacllib/lib64/stub
            ```

        -   当运行环境操作系统架构时arm64时，配置示例如下所示：

            ```
            export DDK_PATH=$HOME/Ascend/ascend-toolkit/latest/arm64-linux
            export NPU_HOST_LIB=$HOME/Ascend/ascend-toolkit/latest/arm64-linux/fwkacllib/lib64/stub
            ```


        ```
        说明：
        使用FwkACLlib组件安装路径下“lib64/stub”目录下的*.so库，是为了编译基于AscendCL接口的代码逻辑时，不依赖其它组件的任何*.so库。编译通过后，在Host上运行应用时，会根据“LD_LIBRARY_PATH”环境变量链接到“fwkacllib/lib64”或者“acllib/lib64”目录下的*.so库，并自动链接到其他组件依赖的*.so库。
        ```
    
    3.  运行环境上，设置运行应用时依赖AscendCL库文件的环境变量。
    
        -   若运行环境上安装的“Ascend-cann-toolkit”包，环境变量设置如下：
    
            ```
            export LD_LIBRARY_PATH=$HOME/Ascend/ascend-toolkit/latest/fwkacllib/lib64
            ```
    
        -   若运行环境上安装的“Ascend-cann-nnrt”包，环境变量设置如下：
    
            ```
            export LD_LIBRARY_PATH=$HOME/Ascend/nnrt/latest/acllib/lib64
            ```
    
        -   若运行环境上安装的“Ascend-cann-nnae”包，环境变量设置如下：
    
            ```
            export LD_LIBRARY_PATH=$HOME/Ascend/nnae/latest/fwkacllib/lib64
            ```


        “$HOME/Ascend”请替换相关软件包的实际安装路径。


-   Ascend 710
    1.  开发环境上，设置生成单算子离线模型的环境变量。

        环境变量配置示例如下：

        ```
        export install_path=$HOME/Ascend/ascend-toolkit/latest
        export PATH=/usr/local/python3.7.5/bin:${install_path}/atc/ccec_compiler/bin:${install_path}/atc/bin:$PATH  # 如果用户环境存在多个python3版本，则指定使用的python3.7.5版本，python3.7.5安装路径请根据实际情况替换；同时设定ATC工具可执行文件所在路径
        export ASCEND_OPP_PATH=${install_path}/opp
        ```

        “$HOME/Ascend”请替换“Ascend-cann-toolkit”包的实际安装路径。

    2.  开发环境上，设置环境变量，配置AscendCL单算子验证程序编译依赖的头文件与库文件路径。

        编译脚本会按环境变量指向的路径查找编译依赖的头文件和库文件，“$HOME/Ascend”请替换“Ascend-cann-toolkit”包的实际安装路径。

        -   当运行环境操作系统架构是x86时，配置示例如下所示：

            ```
            export DDK_PATH=$HOME/Ascend/ascend-toolkit/latest/x86_64-linux
            export NPU_HOST_LIB=$HOME/Ascend/ascend-toolkit/latest/x86_64-linux/acllib/lib64/stub
            ```

        -   当运行环境操作系统架构时arm64时，配置示例如下所示：

            ```
            export DDK_PATH=$HOME/Ascend/ascend-toolkit/latest/arm64-linux
            export NPU_HOST_LIB=$HOME/Ascend/ascend-toolkit/latest/arm64-linux/acllib/lib64/stub
            ```


        ```
        说明：
        使用ACLlib组件安装路径下“lib64/stub”目录下的*.so库，是为了编译基于AscendCL接口的代码逻辑时，不依赖其它组件的任何*.so库。编译通过后，在Host上运行应用时，会根据“LD_LIBRARY_PATH”环境变量链接到“acllib/lib64”目录下的*.so库，并自动链接到其他组件依赖的*.so库。
        ```
    
    3.  运行环境上，设置运行应用时依赖AscendCL库文件的环境变量。
    
        如下为设置环境变量的示例，请将$HOME/Ascend/nnrt/latest替换为Ascend-cann-nnrt包的实际安装路径。
    
        ```
        export LD_LIBRARY_PATH=$HOME/Ascend/nnrt/latest/acllib/lib64
        ```


-   Atlas 200 DK
    1.  开发环境上，设置生成单算子离线模型的环境变量。

        环境变量配置示例如下：

        ```
        export install_path=$HOME/Ascend/ascend-toolkit/latest
        export PATH=/usr/local/python3.7.5/bin:${install_path}/atc/ccec_compiler/bin:${install_path}/atc/bin:$PATH  # 如果用户环境存在多个python3版本，则指定使用的python3.7.5版本，python3.7.5安装路径请根据实际情况替换；同时设定ATC工具可执行文件所在路径
        export ASCEND_OPP_PATH=${install_path}/opp
        ```

        “$HOME/Ascend”请替换“Ascend-cann-toolkit”包的实际安装路径。

    2.  开发环境上，设置环境变量，配置AscendCL单算子验证程序编译依赖的头文件与库文件路径。

        如下为设置环境变量的示例，“$HOME/Ascend”请替换“Ascend-cann-toolkit”包的实际安装路径。

        ```
        export DDK_PATH=$HOME/Ascend/ascend-toolkit/latest/arm64-linux
        export NPU_HOST_LIB=$HOME/Ascend/ascend-toolkit/latest/arm64-linux/acllib/lib64/stub
        ```

        ```
        说明：
        使用ACLlib组件安装路径下“lib64/stub”目录下的*.so库，是为了编译基于AscendCL接口的代码逻辑时，不依赖其它组件的任何*.so库。编译通过后，在Host上运行应用时，会根据“LD_LIBRARY_PATH”环境变量链接到“acllib/lib64”目录下的*.so库，并自动链接到其他组件依赖的*.so库。
        ```

        运行环境上AscendCL运行时依赖的环境变量LD\_LIBRARY\_PATH在制卡时已配置，此处无需单独配置。



## 编译运行（Ascend 310 EP/Ascend 710）<a name="section6961141117311"></a>

1.  生成MatmulTik算子的单算子离线模型文件。
    1.  以运行用户（例如HwHiAiUser）登录开发环境，并进入样例工程的“acl\_execute\_matmul/run/out“目录。
    2.  在out目录下执行如下命令，生成单算子模型文件。

        **atc --singleop=test\_data/config/matmul\_tik\_op.json  --soc\_version=_$\{soc\_version\} _--output=op\_models**

        其中：

        -   singleop：算子描述的json文件。
        -   soc\_version：昇腾AI处理器的型号。
        -   --output=op\_models：代表生成的模型文件存储在当前目录下的op\_models文件夹下。

        模型转换成功后，会生成如下文件：

        在当前目录的op\_models目录下生成单算子的模型文件**0\_MatmulTik\_1\_2\_16\_64\_1\_2\_64\_1024\_1\_2\_16\_1024.om**，命名规范为：序号+opType + 输入的描述\(dateType\_format\_shape\)+输出的描述。

        dataType以及format对应枚举值请从ATC组件所在目录下的“atc/include/graph/types.h”文件中查看，枚举值从0开始依次递增。

        **说明：**模型转换时，会优先去查找自定义算子库去匹配模型文件中的算子。


2.  生成测试数据。

    进入样例工程目录的run/out/test\_data/data目录下，执行如下命令：

    **python3.7.5 generate\_datatik.py**

    会在当前目录下生成数据类型为float16，shape分别为\(16,64\)与\(64,1024\)的数据文件input\_0.bin与input\_1.bin，用于进行MatmulTik算子的验证，并生成预期结果文件np\_result.bin。

3.  编译样例工程，生成单算子验证可执行文件。
    1.  针对Ascend 310，修改src/CMakeLists.txt文件中的如下配置段，将“**acllib**”修改为“**fwkacllib**”，Ascend 710无需执行此步骤。

        ```
        # Header path
        include_directories(
            ${INC_PATH}/acllib/include/
            ../inc/
        )
        ```

    2.  切换到样例工程根目录acl\_execute\_matmul，然后在样例工程根目录下执行如下命令创建目录用于存放编译文件，例如，创建的目录为“build/intermediates/host“。

        **mkdir -p build/intermediates/host**

    3.  切换到“build/intermediates/host”目录，执行cmake命令生成编译文件。

        -   当开发环境与运行环境操作系统架构相同时，执行如下命令编译。

            **cd build/intermediates/host**

            **cmake ../../../src -DCMAKE\_CXX\_COMPILER=g++ -DCMAKE\_SKIP\_RPATH=TRUE**

        -   当开发环境与运行环境操作系统架构不同时，需要使用交叉编译。

            例如，当开发环境为X86架构，运行环境为AArch64架构时，执行以下命令进行交叉编译。

            **cd build/intermediates/host**
            
            **cmake ../../../src -DCMAKE\_CXX\_COMPILER=aarch64-linux-gnu-g++ -DCMAKE\_SKIP\_RPATH=TRUE**


        参数说明如下：
    
        -   “../../../src”表示CMakeLists.txt文件所在的目录，请根据实际目录层级修改。
        -   DCMAKE\_CXX\_COMPILER：编译应用程序所用的编译器。
        -   DCMAKE\_SKIP\_RPATH：**设置为TRUE**，代表不会将rpath信息（即NPU\_HOST\_LIB配置的路径）添加到编译生成的可执行文件中去。可执行文件运行时会自动搜索实际设置的LD\_LIBRARY\_PATH（“xxx/acllib/lib64”或“xxx/fwkacllib/lib64”）中的动态链接库。
    
    4.  执行如下命令，生成可执行文件。
    
        **make**
    
        会在工程目录的“run/out“目录下生成可执行文件**execute\_matmul\_tik\_op**。


4.  在硬件设备的Host侧执行单算子验证文件。
    1.  以运行用户（例如HwHiAiUser）拷贝开发环境中样例工程acl\_execute\_matmul/run/目录下的out文件夹到运行环境（硬件设备Host侧）任一目录，例如上传到/home/HwHiAiUser/HIAI\_PROJECTS/run\_matmul/目录下。

        **说明：**若您的开发环境即为硬件设备的Host侧，此拷贝操作可跳过。

    2.  在运行环境中执行execute\_matmul\_tik\_op文件，验证单算子模型文件。

        在/home/HwHiAiUser/HIAI\_PROJECTS/run\_matmul/out目录下执行如下命令：

        **chmod +x execute\_matmul\_tik\_op**

        **./execute\_matmul\_tik\_op**

        执行完成后，会生成结果二进制文件result\_files/output\_0.bin。

    3.  执行如下命令，将实际结果与预期结果进行比对。

        进入test\_data/data目录下，执行如下命令：

        **python3.7.5 check\_out.py**

        此脚本将算子实际运行结果与预期结果进行比对。

        若回显如下，则代表实际运行结果与预期结果比对成功。

        ```
        Compared with the numpy calculation method, the result is correct.
        ```

        若回显如下，则代表实际运行结果与预期结果比对失败。

        ```
        Compared with the numpy calculation method, the result is wrong.
        ```



## 编译运行（Atlas 200 DK）<a name="section2610307364"></a>

1.  生成MatmulTik算子的单算子离线模型文件。
    1.  以运行用户（例如HwHiAiUser）登录开发环境，并进入样例工程的“acl\_execute\_matmul/run/out“目录。
    2.  在out目录下执行如下命令，生成单算子模型文件。

        **atc --singleop=test\_data/config/matmul\_tik\_op.json  --soc\_version=Ascend310 --output=op\_models**

        其中：

        -   singleop：算子描述的json文件。
        -   soc\_version：昇腾AI处理器的型号。
        -   --output=op\_models：代表生成的模型文件存储在当前目录下的op\_models文件夹下。

        模型转换成功后，会生成如下文件：

        在当前目录的op\_models目录下生成单算子的模型文件**0\_MatmulTik\_1\_2\_16\_64\_1\_2\_64\_1024\_1\_2\_16\_1024.om**，命名规范为：序号+opType + 输入的描述\(dateType\_format\_shape\)+输出的描述。

        dataType以及format对应枚举值请从ATC组件所在目录下的“atc/include/graph/types.h”文件中查看，枚举值从0开始依次递增。

        **说明：**模型转换时，会优先去查找自定义算子库去匹配模型文件中的算子。


2.  生成测试数据。

    进入样例工程目录的run/out/test\_data/data目录下，执行如下命令：

    **python3.7.5 generate\_datatik.py**

    会在当前目录下生成数据类型为float16，shape分别为\(16,64\)与\(64,1024\)的数据文件input\_0.bin与input\_1.bin，用于进行MatmulTik算子的验证，并生成预期结果文件np\_result.bin。

3.  编译样例工程，生成单算子验证可执行文件。
    1.  切换到样例工程根目录acl\_execute\_matmul，然后在样例工程根目录下执行如下命令创建目录用于存放编译文件，例如，创建的目录为“build/intermediates/host“。

        **mkdir -p build/intermediates/host**

    2.  切换到“build/intermediates/host”目录，执行cmake命令生成编译文件。

        **cd build/intermediates/host**

        **cmake ../../../src -DCMAKE\_CXX\_COMPILER=aarch64-linux-gnu-g++ -DCMAKE\_SKIP\_RPATH=TRUE**

        -   “../../../src”表示CMakeLists.txt文件所在的目录，请根据实际目录层级修改。
        -   DCMAKE\_CXX\_COMPILER：编译此应用程序所用的编译器。
        -   DCMAKE\_SKIP\_RPATH：**请设置为TRUE**，代表不会将rpath信息（即NPU\_HOST\_LIB配置的路径）添加到编译生成的可执行文件中去。

            可执行文件运行时会自动搜索实际设置的LD\_LIBRARY\_PATH（“xxx/acllib/lib64”）中的动态链接库。


    3.  执行如下命令，生成可执行文件。
    
        **make**
    
        会在工程目录的“run/out“目录下生成可执行文件**execute\_matmul\_tik\_op**。


4.  在运行环境上执行单算子验证文件。
    1.  以运行用户（例如HwHiAiUser用户）拷贝开发环境中样例工程acl\_execute\_matmul/run/目录下的out文件夹到板端环境任一目录，例如上传到/home/HwHiAiUser/HIAI\_PROJECTS/run\_matmul/目录下。
    2.  在板端环境中执行execute\_matmul\_tik\_op文件，验证单算子模型文件。

        在/home/HwHiAiUser/HIAI\_PROJECTS/run\_matmul/out目录下执行如下命令：

        **chmod +x execute\_matmul\_tik\_op**

        **./execute\_matmul\_tik\_op**

        执行完成后，会屏显出输入数据及输出数据，同时会生成结果二进制文件result\_files/output\_0.bin。

    3.  test\_data/data目录中提供了**check\_out.py**脚本用于比对算子实际结果与调用numpy生成的预期结果。

        开发者可将生成结果文件后的out文件夹拷贝到开发环境，然后在开发环境上进入test\_data/data目录执行如下操作进行结果校验。

        **python3.7.5 check\_out.py**

        此脚本将算子实际运行结果与预期结果进行比对。

        若回显如下，则代表实际运行结果与预期结果比对成功。

        ```
        Compared with the numpy calculation method, the result is correct.
        ```

        若回显如下，则代表实际运行结果与预期结果比对失败。

        ```
        Compared with the numpy calculation method, the result is wrong.
        ```



