# Wlkata开塔桌面级六轴机械臂 + 中国象棋项目开发文档

## 1. 总体功能

本模块为机械臂控制模块，主要功能为接收输入的坐标信息，控制机械臂进行位移、开关气泵，从而实现对棋子的移动。

## 2. 概要设计

象棋中针对棋子的走法只有2种：移动棋子和吃掉对方棋子，进一步分析可将吃子拆解为：先将对方棋子移除棋局，再移动我方棋子。

### 2.1 结构设计

功能由类 RobotController 实现，初始化下棋方（"r"或者"b"）用来选则对应的标定数据，
同时由此配置机械臂对应的端口名(ttyUSB0和ttyUSB1)，并根据标定数据初始化计算映射表。

映射表用来将输入的外部坐标数据转换为机械臂内部坐标系下坐标。
映射表的计算方法如下：
- 根据标定的坐标点，先填充特定行的全部列，再填充行间数据；
- 填充方式是用2个点拟合直线，根据对应XY坐标来回归相应位置的数据。

初始化：
- 通过自带的 robot.home_simultaneous() 成员函数实现复位；
- 用 robot.set_joint_angle() 来将机械臂旋转90°待机，避免与第二台初始化中的机械臂碰撞。

移动棋子 Move() 设计：
- 首先将坐标转换成机械臂内部坐标；
- 通过机械臂的自带库函数 robot.set_wrist_pose() 控制机械臂姿态；
- 移动到指定起始位置后使用 pump_on() 启动气泵吸起棋子；
- 移动到指定落子位置后使用 pump_off() 关闭气泵放下棋子。

吃掉棋子Eat()设计：
- 拆解后改为设计丢弃 Drop() 棋子+移动 Move() 棋子；
- 移动棋子代码可以复用，
- 丢弃棋子用 robot.set_wrist_pose() 将棋子放到棋局外的一旁。

丢弃棋子Drop()设计：
- 移动到指定起始位置后使用 pump_on() 启动气泵吸起棋子；
- 移动机械臂转至棋盘外， 使用 pump_off() 关闭气泵放下棋子。

### 2.2 接口设计

输入：外部调用时主要使用 Move() 和 Eat()，这两者**输入都是起始坐标和落子坐标的整型元组**。
例如 ((700,500),(800,400))。

输出：类不向外返回任何值，只包含部分调试用的打印信息。

### 2.3 数据设计

为了匹配棋盘理解返回的圆心像素坐标，因此特别细化了棋盘格的坐标。标定时尤其要注意。
标定采用1101 x 1001 来映射一个 10 x 9的棋盘格，因此**会有边缘引起的相对坐标变化**。
具体来说，棋盘上的(0, 0)点实际上是标定后的(100, 100)点。
这一点在成员函数 Convert2RobotCoordinate() 中实现。

需要完成标定的信息包括两部分：
- 一部分是确实可以通过开塔软件能走到的位置，这部分的点的XYZ标定数据可以直接读取；包括[100：1000][100：900]
- 另一部分的点由于机械臂存在限位，没有办法移动机械臂到这些位置，全都是凭借附近的点的信息通过等差或者最近邻推断估计的。

## 3. 调用方法

- 确保安装Mirobot包，安装请参考[Mirobot相关代码仓库](http://github.com/wlkata/mirobot-py)
- 将RobotController.py 中的RobotController类import进来；
```
from RobotController import RobotController
```
- 初始化类之后根据业务需求，选择调用成员函数Move(A, B)或者Eat(A, B)

## 4. 补充说明

机械臂由于各种各样的原因（包括但不限于：校准偏差、齿轮背隙、回程误差、使用磨损等）不是那么理想，因此采用了标定的方法来人为除掉这些误差的影响。

不考虑这些因素的理想机械臂，在设计时将坐标系与世界坐标系长度严格对齐，1mm就是坐标系里的1个单位长度，因此，针对棋盘坐标 $(X_c,Y_c)$ 存在简单的线性映射关系，不需要任何标定。

$$X = 105 + Y_c \cdot blockWidth,$$
$$Y = 0 + (4.5 - X_c) \cdot blockWidth,$$
$$Z = constant.$$

其中$blockWidth$是棋盘的格宽，目前项目中采取的方案是18mm。constant为某一常数，正确校准后的机械臂在同一个水平面移动时z轴的参数应当是固定不变的。

在这种前提下，例如我们要抓起棋盘坐标(5, 4)处的棋子，计算得到的机械臂参数就应当是(177, -9, 37)。z轴目前不是很好测量，还没有找到它的基准点是在哪，所以使用了37的经验值。

然而，实际上抓起这个棋子需要的参数值是(180.6, -5, 37.7)。可以看到误差在 3~4mm，这样的误差在象棋项目中是不能接受的，因为棋子本身的半径就不是很大，这样的误差太明显了。

因此使用了标定的方法去人为除掉误差。